@@ -854,8 +854,8 @@ enum reg_class { NO_REGS, R2, R0_1, INDEX_REGS, BASE_REGS, ALL_REGS, LIM_REG_CLA
    in one reasonably fast instruction.  */
 #define MOVE_MAX 65536
 
-/* MOVE_RATIO is the number of move instructions that is better than a
-   block move.  */
+/* If a memory-to-memory move would take MOVE_RATIO or more simple
+   move-instruction pairs, we will do a movstr or libcall instead.  */
 #define MOVE_RATIO 4
 
 /* Define this if zero-extension is slow (more than one real instruction).  */
