@@ -338,8 +338,10 @@ typedef struct rtvec_def{
      REG_SAVE_AREA is used to optimize rtl generated by dynamic stack
    allocations for targets where SETJMP_VIA_SAVE_AREA is true.
      REG_BR_PRED is attached to JUMP_INSNs only, it holds the branch prediction
-   flags computed by get_jump_flags() after dbr scheduling is complete.  */
-
+   flags computed by get_jump_flags() after dbr scheduling is complete.
+     REG_FRAME_RELATED_EXPR is attached to insns that are RTX_FRAME_RELATED_P,
+   but are too complex for DWARF to interpret what they imply.  The attached
+   rtx is used instead of intuition.  */
 
 #define REG_NOTES(INSN)	((INSN)->fld[6].rtx)
 
@@ -352,7 +354,8 @@ enum reg_note { REG_DEAD = 1, REG_INC = 2, REG_EQUIV = 3, REG_WAS_0 = 4,
 		REG_CC_SETTER = 11, REG_CC_USER = 12, REG_LABEL = 13,
 		REG_DEP_ANTI = 14, REG_DEP_OUTPUT = 15, REG_BR_PROB = 16,
 		REG_EXEC_COUNT = 17, REG_NOALIAS = 18, REG_SAVE_AREA = 19,
-		REG_BR_PRED = 20, REG_EH_CONTEXT = 21 };
+		REG_BR_PRED = 20, REG_EH_CONTEXT = 21,
+		REG_FRAME_RELATED_EXPR = 22 };
 /* The base value for branch probability notes.  */
 #define REG_BR_PROB_BASE  10000
 
@@ -1000,6 +1003,8 @@ extern int inequality_comparison_p	PROTO((rtx));
 extern rtx replace_rtx			PROTO((rtx, rtx, rtx));
 extern rtx replace_regs			PROTO((rtx, rtx *, int, int));
 extern int computed_jump_p		PROTO((rtx));
+typedef int (*rtx_function)             PROTO((rtx *, void *));
+extern int for_each_rtx                 PROTO((rtx *, rtx_function, void *));
 
 /* Maximum number of parallel sets and clobbers in any insn in this fn.
    Always at least 3, since the combiner could put that many togetherm
@@ -1050,6 +1055,7 @@ extern struct _global_rtl
   struct rtx_def virtual_stack_vars_val;
   struct rtx_def virtual_stack_dynamic_val;
   struct rtx_def virtual_outgoing_args_val;
+  struct rtx_def virtual_cfa_val;
 } global_rtl;
 
 /* All references to certain hard regs, except those created
@@ -1063,7 +1069,7 @@ extern rtx struct_value_rtx;
 extern rtx struct_value_incoming_rtx;
 extern rtx static_chain_rtx;
 extern rtx static_chain_incoming_rtx;
-
+extern rtx return_address_pointer_rtx;
 
 /* Include the RTL generation functions.  */
 
@@ -1154,7 +1160,17 @@ extern rtx gen_rtx_MEM PROTO((enum machine_mode, rtx));
 
 #define VIRTUAL_OUTGOING_ARGS_REGNUM	((FIRST_VIRTUAL_REGISTER) + 3)
 
-#define LAST_VIRTUAL_REGISTER	((FIRST_VIRTUAL_REGISTER) + 3)
+/* This points to the Canonical Frame Address of the function.  This
+   should corrospond to the CFA produced by INCOMING_FRAME_SP_OFFSET,
+   but is calculated relative to the arg pointer for simplicity; the
+   frame pointer nor stack pointer are necessarily fixed relative to 
+   the CFA until after reload.  */
+
+#define virtual_cfa_rtx			(&global_rtl.virtual_cfa_val)
+
+#define VIRTUAL_CFA_REGNUM		((FIRST_VIRTUAL_REGISTER) + 4)
+
+#define LAST_VIRTUAL_REGISTER		((FIRST_VIRTUAL_REGISTER) + 4)
 
 extern rtx find_next_ref		PROTO((rtx, rtx));
 extern rtx *find_single_use		PROTO((rtx, rtx, rtx *));
@@ -1163,10 +1179,6 @@ extern rtx output_constant_def		PROTO((union tree_node *));
 extern rtx immed_real_const		PROTO((union tree_node *));
 extern union tree_node *make_tree	PROTO((union tree_node *, rtx));
 
-/* Abort routines */
-extern void fatal_insn_not_found	PROTO((rtx));
-extern void fatal_insn			PROTO((char *, rtx));
-
 /* Define a default value for STORE_FLAG_VALUE.  */
 
 #ifndef STORE_FLAG_VALUE
@@ -1338,7 +1350,7 @@ extern void print_inline_rtx		PROTO ((FILE *, rtx, int));
 /* In loop.c */
 extern void init_loop			PROTO ((void));
 #ifdef BUFSIZ
-extern void loop_optimize		PROTO ((rtx, FILE *, int));
+extern void loop_optimize		PROTO ((rtx, FILE *, int, int));
 #endif
 extern void record_excess_regs		PROTO ((rtx, rtx, rtx *));
 
@@ -1403,6 +1415,8 @@ extern int preserve_subexpressions_p	PROTO ((void));
 
 /* In expr.c */
 extern void init_expr_once		PROTO ((void));
+extern void move_by_pieces		PROTO ((rtx, rtx, int, int));
+
 
 /* In stupid.c */
 #ifdef BUFSIZ
